//-----------------------------------------------------------------------------
// Copyright (c) 2007-2008 dhpoware. All Rights Reserved.
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.
//-----------------------------------------------------------------------------

#include <cassert>
#include "GL_EXT_framebuffer_object.h"

// GL_EXT_framebuffer_object

GLboolean glIsRenderbufferEXT(GLuint renderbuffer)
{
    typedef GLboolean (APIENTRY * PFNGLISRENDERBUFFEREXTPROC)(GLuint renderbuffer);
    
    static PFNGLISRENDERBUFFEREXTPROC pfnIsRenderbufferEXT = 0;

    if (!pfnIsRenderbufferEXT)
    {
        pfnIsRenderbufferEXT = reinterpret_cast<PFNGLISRENDERBUFFEREXTPROC>(wglGetProcAddress("glIsRenderbufferEXT"));
        assert(pfnIsRenderbufferEXT != 0);
    }

    return pfnIsRenderbufferEXT(renderbuffer);
}

void glBindRenderbufferEXT(GLenum target, GLuint renderbuffer)
{
    typedef void (APIENTRY * PFNGLBINDRENDERBUFFEREXTPROC)(GLenum target, GLuint renderbuffer);
    
    static PFNGLBINDRENDERBUFFEREXTPROC pfnBindRenderbufferEXT = 0;

    if (!pfnBindRenderbufferEXT)
    {
        pfnBindRenderbufferEXT = reinterpret_cast<PFNGLBINDRENDERBUFFEREXTPROC>(wglGetProcAddress("glBindRenderbufferEXT"));
        assert(pfnBindRenderbufferEXT != 0);
    }
    
    pfnBindRenderbufferEXT(target, renderbuffer);
}

void glDeleteRenderbuffersEXT(GLsizei n, const GLuint *renderbuffers)
{
    typedef void (APIENTRY * PFNGLDELETERENDERBUFFERSEXTPROC)(GLsizei n, const GLuint *renderbuffers);
    
    static PFNGLDELETERENDERBUFFERSEXTPROC pfnDeleteRenderbuffersEXT = 0;

    if (!pfnDeleteRenderbuffersEXT)
    {
        pfnDeleteRenderbuffersEXT = reinterpret_cast<PFNGLDELETERENDERBUFFERSEXTPROC>(wglGetProcAddress("glDeleteRenderbuffersEXT"));
        assert(pfnDeleteRenderbuffersEXT != 0);
    }
    
    pfnDeleteRenderbuffersEXT(n, renderbuffers);
}

void glGenRenderbuffersEXT(GLsizei n, GLuint *renderbuffers)
{
    typedef void (APIENTRY * PFNGLGENRENDERBUFFERSEXTPROC)(GLsizei n, GLuint *renderbuffers);
    
    static PFNGLGENRENDERBUFFERSEXTPROC pfnGenRenderbuffersEXT = 0;

    if (!pfnGenRenderbuffersEXT)
    {
        pfnGenRenderbuffersEXT = reinterpret_cast<PFNGLGENRENDERBUFFERSEXTPROC>(wglGetProcAddress("glGenRenderbuffersEXT"));
        assert(pfnGenRenderbuffersEXT != 0);
    }
    
    pfnGenRenderbuffersEXT(n, renderbuffers);
}

void glRenderbufferStorageEXT(GLenum target, GLenum internalformat, GLsizei width, GLsizei height)
{
    typedef void (APIENTRY * PFNGLRENDERBUFFERSTORAGEEXTPROC)(GLenum target, GLenum internalformat, GLsizei width, GLsizei height);
    
    static PFNGLRENDERBUFFERSTORAGEEXTPROC pfnRenderbufferStorageEXT = 0;

    if (!pfnRenderbufferStorageEXT)
    {
        pfnRenderbufferStorageEXT = reinterpret_cast<PFNGLRENDERBUFFERSTORAGEEXTPROC>(wglGetProcAddress("glRenderbufferStorageEXT"));
        assert(pfnRenderbufferStorageEXT != 0);
    }

    pfnRenderbufferStorageEXT(target, internalformat, width, height);
}

void glGetRenderbufferParameterivEXT(GLenum target, GLenum pname, GLint *params)
{
    typedef void (APIENTRY * PFNGLGETRENDERBUFFERPARAMETERIVEXTPROC)(GLenum target, GLenum pname, GLint *params);
    
    static PFNGLGETRENDERBUFFERPARAMETERIVEXTPROC pfnGetRenderbufferParameterivEXT = 0;

    if (!pfnGetRenderbufferParameterivEXT)
    {
        pfnGetRenderbufferParameterivEXT = reinterpret_cast<PFNGLGETRENDERBUFFERPARAMETERIVEXTPROC>(wglGetProcAddress("glGetRenderbufferParameterivEXT"));
        assert(pfnGetRenderbufferParameterivEXT != 0);
    }

    pfnGetRenderbufferParameterivEXT(target, pname, params);
}

GLboolean glIsFramebufferEXT(GLuint framebuffer)
{
    typedef GLboolean (APIENTRY * PFNGLISFRAMEBUFFEREXTPROC)(GLuint framebuffer);
    
    static PFNGLISFRAMEBUFFEREXTPROC pfnIsFramebufferEXT = 0;

    if (!pfnIsFramebufferEXT)
    {
        pfnIsFramebufferEXT = reinterpret_cast<PFNGLISFRAMEBUFFEREXTPROC>(wglGetProcAddress("glIsFramebufferEXT"));
        assert(pfnIsFramebufferEXT != 0);
    }
    
    return pfnIsFramebufferEXT(framebuffer);
}

void glBindFramebufferEXT(GLenum target, GLuint framebuffer)
{
    typedef void (APIENTRY * PFNGLBINDFRAMEBUFFEREXTPROC)(GLenum target, GLuint framebuffer);
    
    static PFNGLBINDFRAMEBUFFEREXTPROC pfnBindFramebufferEXT = 0;

    if (!pfnBindFramebufferEXT)
    {
        pfnBindFramebufferEXT = reinterpret_cast<PFNGLBINDFRAMEBUFFEREXTPROC>(wglGetProcAddress("glBindFramebufferEXT"));
        assert(pfnBindFramebufferEXT != 0);
    }
    
    pfnBindFramebufferEXT(target, framebuffer);
}

void glDeleteFramebuffersEXT(GLsizei n, const GLuint *framebuffers)
{
    typedef void (APIENTRY * PFNGLDELETEFRAMEBUFFERSEXTPROC)(GLsizei n, const GLuint *framebuffers);
    
    static PFNGLDELETEFRAMEBUFFERSEXTPROC pfnDeleteFramebuffersEXT = 0;

    if (!pfnDeleteFramebuffersEXT)
    {
        pfnDeleteFramebuffersEXT = reinterpret_cast<PFNGLDELETEFRAMEBUFFERSEXTPROC>(wglGetProcAddress("glDeleteFramebuffersEXT"));
        assert(pfnDeleteFramebuffersEXT != 0);
    }
    
    pfnDeleteFramebuffersEXT(n, framebuffers);
}

void glGenFramebuffersEXT(GLsizei n, GLuint *framebuffers)
{
    typedef void (APIENTRY * PFNGLGENFRAMEBUFFERSEXTPROC)(GLsizei n, GLuint *framebuffers);
    
    static PFNGLGENFRAMEBUFFERSEXTPROC pfnGenFramebuffersEXT = 0;

    if (!pfnGenFramebuffersEXT)
    {
        pfnGenFramebuffersEXT = reinterpret_cast<PFNGLGENFRAMEBUFFERSEXTPROC>(wglGetProcAddress("glGenFramebuffersEXT"));
        assert(pfnGenFramebuffersEXT != 0);
    }
    
    pfnGenFramebuffersEXT(n, framebuffers);
}

GLenum glCheckFramebufferStatusEXT(GLenum target)
{
    typedef GLenum (APIENTRY * PFNGLCHECKFRAMEBUFFERSTATUSEXTPROC)(GLenum target);
    
    static PFNGLCHECKFRAMEBUFFERSTATUSEXTPROC pfnCheckFramebufferStatusEXT = 0;

    if (!pfnCheckFramebufferStatusEXT)
    {
        pfnCheckFramebufferStatusEXT = reinterpret_cast<PFNGLCHECKFRAMEBUFFERSTATUSEXTPROC>(wglGetProcAddress("glCheckFramebufferStatusEXT"));
        assert(pfnCheckFramebufferStatusEXT != 0);
    }
    
    return pfnCheckFramebufferStatusEXT(target);
}

void glFramebufferTexture1DEXT(GLenum target, GLenum attachment, GLenum textarget, GLuint texture, GLint level)
{
    typedef void (APIENTRY * PFNGLFRAMEBUFFERTEXTURE1DEXTPROC)(GLenum target, GLenum attachment, GLenum textarget, GLuint texture, GLint level);    
    
    static PFNGLFRAMEBUFFERTEXTURE1DEXTPROC pfnFramebufferTexture1DEXT = 0;

    if (!pfnFramebufferTexture1DEXT)
    {
        pfnFramebufferTexture1DEXT = reinterpret_cast<PFNGLFRAMEBUFFERTEXTURE1DEXTPROC>(wglGetProcAddress("glFramebufferTexture1DEXT"));
        assert(pfnFramebufferTexture1DEXT != 0);
    }
    
    pfnFramebufferTexture1DEXT(target, attachment, textarget, texture, level);
}

void glFramebufferTexture2DEXT(GLenum target, GLenum attachment, GLenum textarget, GLuint texture, GLint level)
{
    typedef void (APIENTRY * PFNGLFRAMEBUFFERTEXTURE2DEXTPROC)(GLenum target, GLenum attachment, GLenum textarget, GLuint texture, GLint level);
    
    static PFNGLFRAMEBUFFERTEXTURE2DEXTPROC pfnFramebufferTexture2DEXT = 0;

    if (!pfnFramebufferTexture2DEXT)
    {
        pfnFramebufferTexture2DEXT = reinterpret_cast<PFNGLFRAMEBUFFERTEXTURE2DEXTPROC>(wglGetProcAddress("glFramebufferTexture2DEXT"));
        assert(pfnFramebufferTexture2DEXT != 0);
    }
    
    pfnFramebufferTexture2DEXT(target, attachment, textarget, texture, level);
}

void glFramebufferTexture3DEXT(GLenum target, GLenum attachment, GLenum textarget, GLuint texture, GLint level, GLint zoffset)
{
    typedef void (APIENTRY * PFNGLFRAMEBUFFERTEXTURE3DEXTPROC)(GLenum target, GLenum attachment, GLenum textarget, GLuint texture, GLint level, GLint zoffset);
    
    static PFNGLFRAMEBUFFERTEXTURE3DEXTPROC pfnFramebufferTexture3DEXT = 0;

    if (!pfnFramebufferTexture3DEXT)
    {
        pfnFramebufferTexture3DEXT = reinterpret_cast<PFNGLFRAMEBUFFERTEXTURE3DEXTPROC>(wglGetProcAddress("glFramebufferTexture3DEXT"));
        assert(pfnFramebufferTexture3DEXT != 0);
    }
    
    pfnFramebufferTexture3DEXT(target, attachment, textarget, texture, level, zoffset);
}

void glFramebufferRenderbufferEXT(GLenum target, GLenum attachment, GLenum renderbuffertarget, GLuint renderbuffer)
{
    typedef void (APIENTRY * PFNGLFRAMEBUFFERRENDERBUFFEREXTPROC)(GLenum target, GLenum attachment, GLenum renderbuffertarget, GLuint renderbuffer);
    
    static PFNGLFRAMEBUFFERRENDERBUFFEREXTPROC pfnFramebufferRenderbufferEXT = 0;

    if (!pfnFramebufferRenderbufferEXT)
    {
        pfnFramebufferRenderbufferEXT = reinterpret_cast<PFNGLFRAMEBUFFERRENDERBUFFEREXTPROC>(wglGetProcAddress("glFramebufferRenderbufferEXT"));
        assert(pfnFramebufferRenderbufferEXT != 0);
    }
    
    pfnFramebufferRenderbufferEXT(target, attachment, renderbuffertarget, renderbuffer);
}

void glGetFramebufferAttachmentParameterivEXT(GLenum target, GLenum attachment, GLenum pname, GLint *params)
{
    typedef void (APIENTRY * PFNGLGETFRAMEBUFFERATTACHMENTPARAMETERIVEXTPROC)(GLenum target, GLenum attachment, GLenum pname, GLint *params);
    
    static PFNGLGETFRAMEBUFFERATTACHMENTPARAMETERIVEXTPROC pfnGetFramebufferAttachmentParameterivEXT = 0;

    if (!pfnGetFramebufferAttachmentParameterivEXT)
    {
        pfnGetFramebufferAttachmentParameterivEXT = reinterpret_cast<PFNGLGETFRAMEBUFFERATTACHMENTPARAMETERIVEXTPROC>(wglGetProcAddress("glGetFramebufferAttachmentParameterivEXT"));
        assert(pfnGetFramebufferAttachmentParameterivEXT != 0);
    }
    
    pfnGetFramebufferAttachmentParameterivEXT(target, attachment, pname, params);
}

void glGenerateMipmapEXT(GLenum target)
{
    typedef void (APIENTRY * PFNGLGENERATEMIPMAPEXTPROC)(GLenum target);
    
    static PFNGLGENERATEMIPMAPEXTPROC pfnGenerateMipmapEXT = 0;

    if (!pfnGenerateMipmapEXT)
    {
        pfnGenerateMipmapEXT = reinterpret_cast<PFNGLGENERATEMIPMAPEXTPROC>(wglGetProcAddress("glGenerateMipmapEXT"));
        assert(pfnGenerateMipmapEXT != 0);
    }
    
    pfnGenerateMipmapEXT(target);
}

// GL_EXT_framebuffer_multisample

void glRenderbufferStorageMultisampleEXT(GLenum target, GLsizei samples, GLenum internalformat, GLsizei width, GLsizei height)
{
    typedef void (APIENTRY * PFNGLRENDERBUFFERSTORAGEMULTISAMPLEEXTPROC)(GLenum target, GLsizei samples, GLenum internalformat, GLsizei width, GLsizei height);

    static PFNGLRENDERBUFFERSTORAGEMULTISAMPLEEXTPROC pfnRenderbufferStorageMultisampleEXT = 0;

    if (!pfnRenderbufferStorageMultisampleEXT)
    {
        pfnRenderbufferStorageMultisampleEXT = reinterpret_cast<PFNGLRENDERBUFFERSTORAGEMULTISAMPLEEXTPROC>(wglGetProcAddress("glRenderbufferStorageMultisampleEXT"));
        assert(pfnRenderbufferStorageMultisampleEXT != 0);
    }

    pfnRenderbufferStorageMultisampleEXT(target, samples, internalformat, width, height);
}

// GL_EXT_framebuffer_blit

void glBlitFramebufferEXT(GLint srcX0, GLint srcY0, GLint srcX1, GLint srcY1, GLint dstX0, GLint dstY0, GLint dstX1, GLint dstY1, GLbitfield mask, GLenum filter)
{
    typedef void (APIENTRY * PFNGLBLITFRAMEBUFFEREXTPROC)(GLint srcX0, GLint srcY0, GLint srcX1, GLint srcY1, GLint dstX0, GLint dstY0, GLint dstX1, GLint dstY1, GLbitfield mask, GLenum filter);

    static PFNGLBLITFRAMEBUFFEREXTPROC pfnBlitFramebufferEXT = 0;

    if (!pfnBlitFramebufferEXT)
    {
        pfnBlitFramebufferEXT = reinterpret_cast<PFNGLBLITFRAMEBUFFEREXTPROC>(wglGetProcAddress("glBlitFramebufferEXT"));
        assert(pfnBlitFramebufferEXT != 0);
    }

    pfnBlitFramebufferEXT(srcX0, srcY0, srcX1, srcY1, dstX0, dstY0, dstX1, dstY1, mask, filter);
}